@@ -81,6 +81,15 @@ class VoilaHandler(JupyterHandler):
         self.kernel_env['SERVER_PORT'] = str(port) if port else ''
         self.kernel_env['SERVER_NAME'] = host
 
+        # Add HTTP Headers as env vars following rfc3875#section-4.1.18
+        if len(self.voila_configuration.http_header_envs) > 0:
+            config_headers_lower = [header.lower() for header in self.voila_configuration.http_header_envs]
+            for header_name in self.request.headers:
+                # Use case insensitive comparison of header names as per rfc2616#section-4.2
+                if header_name.lower() in config_headers_lower:
+                    env_name = f'HTTP_{header_name.upper().replace("-", "_")}'
+                    self.kernel_env[env_name] = self.request.headers.get(header_name)
+
         # we can override the template via notebook metadata or a query parameter
         template_override = None
         if 'voila' in notebook.metadata and self.voila_configuration.allow_template_override in ['YES', 'NOTEBOOK']:
@@ -153,7 +162,7 @@ class VoilaHandler(JupyterHandler):
         # render notebook in snippets, and flush them out to the browser can render progresssively
         async for html_snippet, resources in self.exporter.generate_from_notebook_node(notebook, resources=resources, extra_context=extra_context):
             self.write(html_snippet)
-            self.flush()  # we may not want to consider not flusing after each snippet, but add an explicit flush function to the jinja context
+            self.flush()  # we may not want to consider not flushing after each snippet, but add an explicit flush function to the jinja context
             # yield  # give control back to tornado's IO loop, so it can handle static files or other requests
         self.flush()
 
@@ -191,6 +200,8 @@ class VoilaHandler(JupyterHandler):
         # see the updated variable (it seems to be local to our block)
         nb.cells = result.cells
 
+        await self._cleanup_resources()
+
     async def _jinja_cell_generator(self, nb, kernel_id):
         """Generator that will execute a single notebook cell at a time"""
         nb, resources = ClearOutputPreprocessor().preprocess(nb, {'metadata': {'path': self.cwd}})
@@ -241,8 +252,14 @@ class VoilaHandler(JupyterHandler):
             finally:
                 yield output_cell
 
+        await self._cleanup_resources()
+
+    async def _cleanup_resources(self):
+        await ensure_async(self.executor.km.cleanup_resources())
+        await ensure_async(self.executor.kc.stop_channels())
+
     async def load_notebook(self, path):
-        model = self.contents_manager.get(path=path)
+        model = await ensure_async(self.contents_manager.get(path=path))
         if 'content' not in model:
             raise tornado.web.HTTPError(404, 'file not found')
         __, extension = os.path.splitext(model.get('path', ''))
